#include "ikk-pwm.h"
#include "tl_common.h"
#include "ikk-led.h"
#include "../general/ikk-debug.h"


#define PWM_UNUSED_ID					PWM_UNKNOW_ID

#define PWM_TICK_LOOP_NMS				10	// MS
// led controller
static struct tag_pwm_controller_st {
	uint32_t actionCounter; //event couter
	uint32_t intervalDutyCycle; //+ / - every times
	uint32_t durationMs;
	uint16_t startDutyCycle;
	uint16_t endDutyCycle;
	uint16_t startDutyCycleValue;
	PwmGpioSt gpioInfo;
}g_stPwmCtrller[PWM_OBJS_SUPPORT_MAXN];
static PwmGpioListSt stPWM_Mutex_Info[] = PWM_TLSR8258_INFO_INIT;

static PwmConfSt g_stPwmConfList[PWM_OBJS_SUPPORT_MAXN];
static uint8_t g_u8PwmChannelNum = 0;

pPwmActionDoneCallback g_pPwmActionDoneCallback = NULL;

static bool Pwm_Tick_Detect_Running_Flag  = false;

static ev_timer_event_t *Tick_Pwm_TimerEvt = NULL;


static s32 kPwmGradientChangeEventHandler(void *data);



static PwmChannelEnum Get_PWM_CHANNEL_FROM_PIN(uint32_t pin)
{
	for(uint8_t index = 0;index < PWM_OBJS_SUPPORT_MAXN;index++ )
		for(uint8_t i = 0;i<2;i++)
			for(uint8_t j=0;j< stPWM_Mutex_Info[index].gpioinfo[i].vaild_num;j++)
				if(stPWM_Mutex_Info[index].gpioinfo[i].pin[j] == pin)
					return stPWM_Mutex_Info[index].pwmChannel;
	return 0xff;
}
static GPIO_FuncTypeDef Get_PIN_FUNC_By_PIN(uint32_t pin)
{
	for(uint8_t index = 0;index < PWM_OBJS_SUPPORT_MAXN;index++ )
		for(uint8_t i = 0;i<2;i++)
			for(uint8_t j=0;j< stPWM_Mutex_Info[index].gpioinfo[i].vaild_num;j++)
				if(stPWM_Mutex_Info[index].gpioinfo[i].pin[j] == pin)
					return stPWM_Mutex_Info[index].gpioinfo[i].Pin_Funs;
	return 0xff;
}

void pwmSetDuty(u8 ch, u16 dutycycle)
{
	uint32_t ticksPerPeriod = CLOCK_SYS_CLOCK_HZ / PWM_FREQUENCY;
	u32 cmp_tick = ((u32)dutycycle * ticksPerPeriod) / ( PWM_TIMER_MAX_DUTY_CYCLE_NUM);
	drv_pwm_cfg(ch, (u16)cmp_tick, ticksPerPeriod);
}
/*	DESP: get the pwm index by led_id.
 * 	Auth: maozj.20200211.
 * */
uint8_t kPwmGetIndexByID(uint8_t pwm_id )
{
	for(int index = 0; index < g_u8PwmChannelNum; ++index ) {
		//iKonkeAfSelfPrint("####kPwmGetIndexByID index[%d] pwm_id[%d]\r\n",index,  g_stPwmConfList[index].u8PwmId);
		if( g_stPwmConfList[index].u8PwmId != PWM_UNUSED_ID ) {
			if( g_stPwmConfList[index].u8PwmId == pwm_id ) {
				return index;
			}
		}
		else break;
	}

	return 0XFF;
}
/*	DESP: registration the system pwm driver processing function interface.
	Auth: maozj.20200211.
*/
void kPwmDriverhandler(uint8_t pwm_index, PwmGradientDirectionEnum st, uint16_t value)
{
	Pwm_Tick_Detect_Running_Flag = false;

	if (pwm_index >= g_u8PwmChannelNum){
		iKonkeAfSelfPrint("####kPwmDriverhandler index is too large\r\n");
		return;
	}
	//iKonkeAfSelfPrint("######kPwmDriverhandler index(%d), direction(%d), value(%d)\r\n", pwm_index, st, value);

	switch(st) {
		case (ELP_MIN):
		{
			pwmSetDuty(g_stPwmConfList[pwm_index].gpioInfo.pwmChannel,PWM_TIMER_MIN_DUTY_CYCLE_NUM);
			break;
		}
		case (ELP_MID):
		{
			pwmSetDuty(g_stPwmConfList[pwm_index].gpioInfo.pwmChannel,PWM_TIMER_MID_DUTY_CYCLE_NUM);
			break;
		}

		case (ELP_ANY):
		{
			//must use param[value] when pwm set a value
			pwmSetDuty(g_stPwmConfList[pwm_index].gpioInfo.pwmChannel,value);
			break;
		}
		case (ELP_MAX):
		{
			pwmSetDuty(g_stPwmConfList[pwm_index].gpioInfo.pwmChannel,PWM_TIMER_MAX_DUTY_CYCLE_NUM);
			break;
		}
		case (ELP_TO_BRIGHT):
		case (ELP_TO_DARKEN):
		{
			break;
		}
		default: break;
	}
}
/*	DESP: Check whether all pwm have stopped action detection.
 * 	Auth: maozj.20200211.
 * */
bool kPwmGradientChangeIsGoing(void )
{
	for(int pwm_index = 0; pwm_index < g_u8PwmChannelNum; ++pwm_index ) {
		if( g_stPwmCtrller[pwm_index].actionCounter > 0 ) {
			return true;
		}
		//else break;
	}

	return false;
}

/*	DESP: Check if the specified pwm  is changing.
 * 	Auth: maozj.20200211.
 * */
bool kPwmIsChanging(uint8_t pwm_id )
{
	uint8_t pwm_index = kPwmGetIndexByID(pwm_id);

	if( pwm_index != 0XFF ) {
		return (g_stPwmCtrller[pwm_index].actionCounter > 0);
	}

	return false;
}

/*	DESP: Indicator lamp behavior control interface.
	Auth: dingmz_frc.20191108.
*/
void kPwmOptTrigger(uint8_t id, uint32_t start_value, uint32_t end_value, uint32_t duration_time )
{
	uint8_t pwm_index = kPwmGetIndexByID(id);

	if( pwm_index == 0XFF || (start_value == end_value) ) {
		iKonkeAfSelfPrint("#####PWM value is invalid index(%d)id(%d)start(%d)end(%d) \r\n", pwm_index, id, start_value, end_value);
		return ;
	}

	iKonkeAfSelfPrint("PWM_TRIGGER: pwm_index[%d], id(%d), start_value(%d), end_value(%d), times(%d)\r\n",
			pwm_index, id, start_value, end_value, duration_time);

	g_stPwmCtrller[pwm_index].actionCounter  = duration_time / PWM_TICK_LOOP_NMS;
	g_stPwmCtrller[pwm_index].intervalDutyCycle = absSub(start_value,end_value) \
													/ g_stPwmCtrller[pwm_index].actionCounter;

	g_stPwmCtrller[pwm_index].startDutyCycle = start_value;
	g_stPwmCtrller[pwm_index].endDutyCycle = end_value;
	g_stPwmCtrller[pwm_index].startDutyCycleValue = start_value;
	memcpy(&g_stPwmCtrller[pwm_index].gpioInfo, &g_stPwmConfList[pwm_index].gpioInfo, sizeof(PwmGpioSt));
	iKonkeAfSelfPrint("action (%d) , interval[%d]\r\n",g_stPwmCtrller[pwm_index].actionCounter,g_stPwmCtrller[pwm_index].intervalDutyCycle);
	pwmSetDuty(g_stPwmConfList[pwm_index].gpioInfo.pwmChannel,start_value);

	if(!Pwm_Tick_Detect_Running_Flag) {
		Pwm_Tick_Detect_Running_Flag = true;
	}
}
/*	DESP: single LED instance register interface.
 * 	Auth: dingmz_frc.20191108.
 * */
kk_err_t kPwmRegister(uint8_t pwm_id, PwmGpioSt gpioInfo )
{
	if( PWM_UNKNOW_ID == pwm_id || PWM_ALLOPT_ID == pwm_id ) {
		return KET_ERR_INVALID_PARAM;
	}

	int valid_index = PWM_OBJS_SUPPORT_MAXN;
	kk_err_t err = KET_OK;
	for(int index = 0; index < PWM_OBJS_SUPPORT_MAXN; ++index ) {
		if( g_stPwmConfList[index].u8PwmId == pwm_id ) {
			valid_index = index;
			break;
		}else if((valid_index == PWM_OBJS_SUPPORT_MAXN) && (g_stPwmConfList[index].u8PwmId == PWM_UNUSED_ID)) {
			valid_index = index;
		}
	}
	uint8_t channel = Get_PWM_CHANNEL_FROM_PIN(gpioInfo.pin);
	if(channel == 0xff)
	{
		iKonkeAfSelfPrint("pin (%d) not support pwm function!\r\n",gpioInfo.pin);
		return KET_ERR_CONTENT_MISMATCH;
	}

	if( valid_index < PWM_OBJS_SUPPORT_MAXN ) {
		memset(&g_stPwmCtrller[valid_index], 0, sizeof(g_stPwmCtrller[valid_index]));

		g_stPwmConfList[valid_index].u8PwmId = pwm_id;
		g_stPwmConfList[valid_index].gpioInfo.pin = gpioInfo.pin;
		g_stPwmConfList[valid_index].gpioInfo.pwmChannel = channel;

		gpio_set_func(g_stPwmConfList[valid_index].gpioInfo.pin,Get_PIN_FUNC_By_PIN(gpioInfo.pin));
		iKonkeAfSelfPrint("PWM_REGISTER:index(%d),id(%d), pwmChannel(%d)\r\n",valid_index,  g_stPwmConfList[valid_index].u8PwmId
			, g_stPwmConfList[valid_index].gpioInfo.pwmChannel);
		pwmSetDuty(g_stPwmConfList[valid_index].gpioInfo.pwmChannel,0);
		drv_pwm_start(g_stPwmConfList[valid_index].gpioInfo.pwmChannel);
	}else {
		err = KET_ERR_INSUFFICIENT_SPACE;
	}

	return err;
}
/*	DESP: led control module Initialize configuration interface.
 * 	Note: Led initialization interface supports one-time registration of multiple LEDs!
 * 	Auth: dingmz_frc.20191108.
 * */
kk_err_t kPwmModuleInit(PwmConfSt pwmlist[], unsigned char pwm_number, pPwmActionDoneCallback pCallback)
{
	kk_err_t err = KET_OK;

	memset(g_stPwmConfList, PWM_UNUSED_ID, sizeof(g_stPwmConfList));
	memset(g_stPwmCtrller, 0, sizeof(g_stPwmCtrller));

	if(pwmlist && (pwm_number > 0)) {
		g_u8PwmChannelNum = (pwm_number < PWM_OBJS_SUPPORT_MAXN)?pwm_number:PWM_OBJS_SUPPORT_MAXN;

		for(int index = 0; index < g_u8PwmChannelNum; ++index ) {
			err = kPwmRegister(pwmlist[index].u8PwmId, pwmlist[index].gpioInfo);
		}

	}
	//init callback function when led blink over
	g_pPwmActionDoneCallback = pCallback;

	//init software timer
	if(Tick_Pwm_TimerEvt){
		TL_ZB_TIMER_CANCEL(&Tick_Pwm_TimerEvt);
	}
	Tick_Pwm_TimerEvt = TL_ZB_TIMER_SCHEDULE(kPwmGradientChangeEventHandler, NULL, PWM_TICK_LOOP_NMS);
	return err;
}
static s32 kPwmGradientChangeEventHandler(void *data)
{
	if(!Pwm_Tick_Detect_Running_Flag)
		return 0;
	for (uint8_t i = 0; i < g_u8PwmChannelNum; i++){
		if (g_stPwmCtrller[i].actionCounter > 0){
			if (g_stPwmCtrller[i].startDutyCycle > g_stPwmCtrller[i].endDutyCycle){
				g_stPwmCtrller[i].actionCounter--;
				g_stPwmCtrller[i].startDutyCycleValue -= g_stPwmCtrller[i].intervalDutyCycle;
				pwmSetDuty(g_stPwmCtrller[i].gpioInfo.pwmChannel,g_stPwmCtrller[i].startDutyCycleValue);

			} else if (g_stPwmCtrller[i].startDutyCycle < g_stPwmCtrller[i].endDutyCycle){
				g_stPwmCtrller[i].actionCounter--;
				g_stPwmCtrller[i].startDutyCycleValue += g_stPwmCtrller[i].intervalDutyCycle;
				pwmSetDuty(g_stPwmCtrller[i].gpioInfo.pwmChannel,g_stPwmCtrller[i].startDutyCycleValue);
			}
			//iKonkeAfSelfPrint("BREATH index(%d)channel(%d) count(%d),%d\r\n", i,g_stPwmCtrller[i].gpioInfo.pwmChannel, g_stPwmCtrller[i].actionCounter,g_stPwmCtrller[i].startDutyCycleValue);
			//PWM_IDص
			if (g_stPwmCtrller[i].actionCounter == 0){
				//may be end duty cycle not equal, restore end duty cycle
				pwmSetDuty(g_stPwmCtrller[i].gpioInfo.pwmChannel,g_stPwmCtrller[i].endDutyCycle);

				if (g_pPwmActionDoneCallback){
					PwmGradientDirectionEnum opt = ELP_OPT_DEFAULT;
					if (g_stPwmCtrller[i].startDutyCycle > g_stPwmCtrller[i].endDutyCycle){
						opt = ELP_TO_DARKEN;
					}else if (g_stPwmCtrller[i].startDutyCycle < g_stPwmCtrller[i].endDutyCycle){
						opt = ELP_TO_BRIGHT;
					}

					if (opt != ELP_OPT_DEFAULT){
						//user custom  implement
						g_pPwmActionDoneCallback(g_stPwmConfList[i].u8PwmId, opt);
					}
				}
			}
		}
	}
	return 0;
}
/*	DESP: clear pwm gradient counter interface by pwm id.

 * 	Auth: maozj.20200212.
 * */
bool kPwmClearGradientCounterById(uint8_t pwm_id)
{
	if (pwm_id == PWM_UNUSED_ID){
		iKonkeAfSelfPrint("#Err: kPwmClearGradientCounterById pwm id is invalid\r\n");
		return false;
	}

	for (int index = 0; index < PWM_OBJS_SUPPORT_MAXN; ++index ) {
		//iKonkeAfSelfPrint("####kPwmGetIndexByID index[%d] pwm_id[%d]\r\n",index,  g_stPwmConfList[index].u8PwmId);

		if (g_stPwmConfList[index].u8PwmId == pwm_id) {
			iKonkeAfSelfPrint("###kPwmClearGradientCounterById success index(%d) id(%d)\r\n", index, pwm_id);
			g_stPwmCtrller[index].actionCounter = 0;
			return true;
		}
	}

	return false;
}

void PWM_Test(uint8_t data)
{
//	gpio_set_func(GPIO_PB3,AS_PWM0_N);
//	if(data == 1)
//		pwmSetDuty(0,60000);
//	else
//		pwmSetDuty(0,0);
//	drv_pwm_start(0);
//	for(uint8_t index = 0;index < PWM_OBJS_SUPPORT_MAXN;index++ )
//	{
//		iKonkeAfSelfPrint("stPWM_Mutex_Info(%d)\r\n",index);
//		for(uint8_t i = 0;i<2;i++)
//		{
//			iKonkeAfSelfPrint("gpio(%d),num = %d\r\n",i,stPWM_Mutex_Info[index].gpioinfo[i].vaild_num);
//			for(uint8_t j=0;j< stPWM_Mutex_Info[index].gpioinfo[i].vaild_num;j++)
//			{
//				iKonkeAfSelfPrint("gpio(%d)\r\n",stPWM_Mutex_Info[index].gpioinfo[i].pin[j]);
//			}
//		}
//	}

}
